package com.epson.epos2_cat;

import android.os.Bundle;
import android.support.annotation.NonNull;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.TextView;

import com.epson.epos2.cat.AccessDailyLogListener;
import com.epson.epos2.cat.AuthorizeCompletionListener;
import com.epson.epos2.cat.AuthorizeRefundListener;
import com.epson.epos2.cat.AuthorizeResult;
import com.epson.epos2.cat.AuthorizeSalesListener;
import com.epson.epos2.cat.AuthorizeVoidListener;
import com.epson.epos2.cat.Cat;
import com.epson.epos2.cat.CheckConnectionListener;
import com.epson.epos2.cat.ClearOutputListener;
import com.epson.epos2.cat.ScanCodeListener;
import com.epson.epos2.cat.DailyLog;
import com.epson.epos2.cat.DirectIOListener;
import com.epson.epos2.cat.Epos2CallbackCode;

import java.util.ArrayList;

import static com.epson.epos2_cat.MainActivity.mCat;
import static com.epson.epos2_cat.MainActivity.mContext;

public class AuthorizeFragment extends CatFragment implements View.OnClickListener, AccessDailyLogListener, AuthorizeCompletionListener, AuthorizeRefundListener, AuthorizeSalesListener, AuthorizeVoidListener,CheckConnectionListener, ClearOutputListener, ScanCodeListener, DirectIOListener {
    private Spinner mSpnService = null;
    private Spinner mSpnAuthorize = null;
    private EditText mTextTotalAmount = null;
    private EditText mTextAmount = null;
    private EditText mTextTax = null;
    private EditText mTextAsi = null;
    private EditText mTextDailyLogType = null;
    private TextView mTextCat = null;

    private static final int SEQUENCE_NUM = 0001;
    private static final int AuthorizeSales = 0;
    private static final int AuthorizeVoid = 1;
    private static final int AuthorizeRefund = 2;
    private static final int AuthorizeCompletion = 3;
    private static final int ClearOutput = 4;

    public static AuthorizeFragment newInstance() {
        return new AuthorizeFragment();
    }

    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {

        View rootView = inflater.inflate(R.layout.fragment_authorize, container, false);

        mSpnService = rootView.findViewById(R.id.spnService);
        ArrayAdapter<SpnItems> endModeAdapter = new ArrayAdapter<SpnItems>(getActivity(), android.R.layout.simple_spinner_item);
        endModeAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        endModeAdapter.add(new SpnItems(getString(R.string.service_credit), Cat.SERVICE_CREDIT));
        endModeAdapter.add(new SpnItems(getString(R.string.service_debit), Cat.SERVICE_DEBIT));
        endModeAdapter.add(new SpnItems(getString(R.string.service_unionpay), Cat.SERVICE_UNIONPAY));
        endModeAdapter.add(new SpnItems(getString(R.string.service_edy), Cat.SERVICE_EDY));
        endModeAdapter.add(new SpnItems(getString(R.string.service_id), Cat.SERVICE_ID));
        endModeAdapter.add(new SpnItems(getString(R.string.service_nanaco), Cat.SERVICE_NANACO));
        endModeAdapter.add(new SpnItems(getString(R.string.service_quicpay), Cat.SERVICE_QUICPAY));
        endModeAdapter.add(new SpnItems(getString(R.string.service_suica), Cat.SERVICE_SUICA));
        endModeAdapter.add(new SpnItems(getString(R.string.service_waon), Cat.SERVICE_WAON));
        endModeAdapter.add(new SpnItems(getString(R.string.service_point), Cat.SERVICE_POINT));
        endModeAdapter.add(new SpnItems(getString(R.string.service_nfcpayment), Cat.SERVICE_NFCPAYMENT));
        endModeAdapter.add(new SpnItems(getString(R.string.service_pitapa), Cat.SERVICE_PITAPA));
        endModeAdapter.add(new SpnItems(getString(R.string.service_fisc), Cat.SERVICE_FISC));
        endModeAdapter.add(new SpnItems(getString(R.string.service_qr), Cat.SERVICE_QR));
        mSpnService.setAdapter(endModeAdapter);
        mSpnService.setSelection(0);

        mTextTotalAmount = rootView.findViewById(R.id.editText_TotalAmount);
        mTextAmount = rootView.findViewById(R.id.editText_Amount);
        mTextTax = rootView.findViewById(R.id.editText_Tax);
        mTextDailyLogType = rootView.findViewById(R.id.editText_DailyLogType);
        mTextAsi = rootView.findViewById(R.id.editText_asiAuthorize);

        mSpnAuthorize = (Spinner) rootView.findViewById(R.id.spnAuthorize);
        ArrayAdapter<SpnItems> authorizeAdapter = new ArrayAdapter<SpnItems>(getActivity(), android.R.layout.simple_spinner_item);
        authorizeAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        authorizeAdapter.add(new SpnItems(getString(R.string.authorize_sales), AuthorizeSales));
        authorizeAdapter.add(new SpnItems(getString(R.string.authorize_void), AuthorizeVoid));
        authorizeAdapter.add(new SpnItems(getString(R.string.authorize_refund), AuthorizeRefund));
        authorizeAdapter.add(new SpnItems(getString(R.string.authorize_completion), AuthorizeCompletion));
        authorizeAdapter.add(new SpnItems(getString(R.string.clearOutput), ClearOutput));
        mSpnAuthorize.setAdapter(authorizeAdapter);
        mSpnAuthorize.setSelection(0);

        Button btn_clearance = rootView.findViewById(R.id.button_clearance);
        btn_clearance.setOnClickListener(this);

        Button btn_accessDailyLog = rootView.findViewById(R.id.button_accessDailyLog);
        btn_accessDailyLog.setOnClickListener(this);

        Button btn_checkConnection = rootView.findViewById(R.id.button_checkConnection);
        btn_checkConnection.setOnClickListener(this);

        Button btn_clear = rootView.findViewById(R.id.button_clear);
        btn_clear.setOnClickListener(this);

        Button btn_scanCode = rootView.findViewById(R.id.button_scanCode);
        btn_scanCode.setOnClickListener(this);

        mTextCat = rootView.findViewById(R.id.textView_commandText);

        super.setTextView(mTextCat);

        return rootView;
    }

    @Override
    public void setUserVisibleHint(boolean isVisibleToUser) {
        super.setUserVisibleHint(isVisibleToUser);
        if (isVisibleToUser) {
            if (mCat != null) {
                mCat.setAuthorizeSalesEventListener(this);
                mCat.setAuthorizeVoidEventListener(this);
                mCat.setClearOutputEventListener(this);
                mCat.setAuthorizeRefundEventListener(this);
                mCat.setAuthorizeCompletionEventListener(this);
                mCat.setAccessDailyLogEventListener(this);
                mCat.setCheckConnectionEventListener(this);
                mCat.setScanCodeEventListener(this);
                mCat.setDirectIOEventListener(this);
            }
            super.setTextView(mTextCat);
        } else {
            if (mCat != null) {
                mCat.setAuthorizeSalesEventListener(null);
                mCat.setAuthorizeVoidEventListener(null);
                mCat.setClearOutputEventListener(null);
                mCat.setAuthorizeRefundEventListener(null);
                mCat.setAuthorizeCompletionEventListener(null);
                mCat.setAccessDailyLogEventListener(null);
                mCat.setCheckConnectionEventListener(null);
                mCat.setScanCodeEventListener(null);
                mCat.setDirectIOEventListener(null);
            }
        }
    }

    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.button_clearance:
                onCrealance();
                break;
            case R.id.button_accessDailyLog:
                onAccessDailyLog();
                break;
            case R.id.button_checkConnection:
                onCheckConnection();
                break;
            case R.id.button_scanCode:
                onScanCode();
                break;
            case R.id.button_clear:
                onClearCat();
                break;
            default:
                // Do nothing
                break;
        }
    }

    private void onCrealance() {
        if (mCat == null) {
            return;
        }

        String methodName = "";
        try {
            int authorize = ((SpnItems) mSpnAuthorize.getSelectedItem()).getItemConstant();
            int service = ((SpnItems) mSpnService.getSelectedItem()).getItemConstant();
            int totalAmount = Integer.parseInt(mTextTotalAmount.getText().toString());
            int amount = Integer.parseInt(mTextAmount.getText().toString());
            int tax = Integer.parseInt(mTextTax.getText().toString());
            String asi = mTextAsi.getText().toString();
            switch (authorize) {
                case AuthorizeSales:
                    methodName += "authorizeSales";
                    mCat.authorizeSales(service, totalAmount, amount, tax, SEQUENCE_NUM, asi);
                    break;
                case AuthorizeVoid:
                    methodName += "authorizeVoid";
                    mCat.authorizeVoid(service, totalAmount, amount, tax, SEQUENCE_NUM, asi);
                    break;
                case AuthorizeRefund:
                    methodName += "authorizeRefund";
                    mCat.authorizeRefund(service, totalAmount, amount, tax, SEQUENCE_NUM, asi);
                    break;
                case AuthorizeCompletion:
                    methodName += "authorizeCompletion";
                    mCat.authorizeCompletion(service, totalAmount, amount, tax, SEQUENCE_NUM, asi);
                    break;
                case ClearOutput:
                    methodName += "clearOutput";
                    mCat.clearOutput();
                    break;
                default:
                    break;
            }
        } catch (Exception e) {
            ShowMsg.showException(e, methodName, mContext);
        }
    }

    private void onAccessDailyLog() {
        if (mCat == null) {
            return;
        }
        int service = ((SpnItems) mSpnService.getSelectedItem()).getItemConstant();
        String dailyLogType =  mTextDailyLogType.getText().toString();
        String asi = mTextAsi.getText().toString();

        try {
            mCat.accessDailyLog(service, SEQUENCE_NUM, dailyLogType, asi);
        } catch (Exception e) {
            ShowMsg.showException(e, "accessDailyLog", mContext);
        }
    }

    private void onCheckConnection() {
        if (mCat == null) {
            return;
        }

        String asi = mTextAsi.getText().toString();

        try {
            mCat.checkConnection(asi);
        } catch (Exception e) {
            ShowMsg.showException(e, "checkConnection", mContext);
        }
    }

    private void onScanCode() {
        if (mCat == null) {
            return;
        }

        try {
            mCat.scanCode();
        } catch (Exception e) {
            ShowMsg.showException(e, "scanCode", mContext);
        }
    }

    private void onClearCat() {
        getActivity().runOnUiThread(new Runnable() {
            @Override
            public synchronized void run() {
                mTextCat.setText("");
            }
        });
    }

    private String makeAuthorizeResultMessage(AuthorizeResult result) {
        String resultMsg = "";
        if (result != null) {
            resultMsg += "    AccountNumber:" + result.getAccountNumber() + "\n";
            resultMsg += "    SettledAmount:" + result.getSettledAmount() + "\n";
            resultMsg += "    SlipNumber:" + result.getSlipNumber() + "\n";
            resultMsg += "    Kid:" + result.getKid() + "\n";
            resultMsg += "    ApprovalCode:" + result.getApprovalCode() + "\n";
            resultMsg += "    TransactionNumber:" + result.getTransactionNumber() + "\n";
            resultMsg += "    PaymentCondition:" + super.getPaymentConditionText(result.getPaymentCondition()) + "\n";
            resultMsg += "    VoidSlipNumber:" + result.getVoidSlipNumber() + "\n";
            resultMsg += "    Balance:" + result.getBalance() + "\n";
            resultMsg += "    TransactionType:" + result.getTransactionType() + "\n";
            resultMsg += "    AdditionalSecurityInformation:" + result.getAdditionalSecurityInformation() + "\n";
        }
        return resultMsg;
    }

    @Override
    public void onCatAuthorizeSales(final Cat catObj, final int code, final int sequence, final int service, final AuthorizeResult result) {
        getActivity().runOnUiThread(new Runnable() {
            @Override
            public synchronized void run() {
                int oposCode = 0;
                if (code == Epos2CallbackCode.CODE_ERR_OPOSCODE) {
                    oposCode = catObj.getOposErrorCode();
                    ShowMsg.showResult(code, oposCode, mContext);
                } else {
                    ShowMsg.showResult(code, "", mContext);
                }

                String txtCat = "";
                txtCat += "OnCatAuthorizeSales:\n";
                txtCat += "  Sequence:" + sequence + "\n";
                txtCat += "  Service:" + getCatServiceText(service) + "\n";
                txtCat += "  Result:" + "\n" + makeAuthorizeResultMessage(result);
                mTextCat.append(txtCat);
            }
        });
    }

    @Override
    public void onCatAuthorizeVoid(final Cat catObj, final int code, final int sequence, final int service, final AuthorizeResult result) {
        getActivity().runOnUiThread(new Runnable() {
            @Override
            public synchronized void run() {
                int oposCode = 0;
                if (code == Epos2CallbackCode.CODE_ERR_OPOSCODE) {
                    oposCode = catObj.getOposErrorCode();
                    ShowMsg.showResult(code, oposCode, mContext);
                } else {
                    ShowMsg.showResult(code, "", mContext);
                }

                String txtCat = "";
                txtCat += "OnCatAuthorizeVoid:\n";
                txtCat += "  Sequence:" + sequence + "\n";
                txtCat += "  Service:" + getCatServiceText(service) + "\n";
                txtCat += "  Result:" + "\n" + makeAuthorizeResultMessage(result);
                mTextCat.append(txtCat);
            }
        });
    }

    @Override
    public void onCatClearOutput(final Cat catObj, final int code, final int abortCode) {
        getActivity().runOnUiThread(new Runnable() {
            @Override
            public synchronized void run() {
                int oposCode = 0;
                if (code == Epos2CallbackCode.CODE_ERR_OPOSCODE) {
                    oposCode = catObj.getOposErrorCode();
                    ShowMsg.showResult(code, oposCode, mContext);
                } else {
                    ShowMsg.showResult(code, "", mContext);
                }

                String txtCat = "";
                txtCat += "OnCatClearOutput:\n";
                txtCat += "  AbortCode:" + abortCode + "\n";
                mTextCat.append(txtCat);
            }
        });
    }

    @Override
    public void onCatAuthorizeRefund(final Cat catObj, final int code, final int sequence, final int service, final AuthorizeResult result) {
        getActivity().runOnUiThread(new Runnable() {
            @Override
            public synchronized void run() {
                int oposCode = 0;
                if (code == Epos2CallbackCode.CODE_ERR_OPOSCODE) {
                    oposCode = catObj.getOposErrorCode();
                    ShowMsg.showResult(code, oposCode, mContext);
                } else {
                    ShowMsg.showResult(code, "", mContext);
                }

                String txtCat = "";
                txtCat += "OnCatAuthorizeRefund:\n";
                txtCat += "  Sequence:" + sequence + "\n";
                txtCat += "  Service:" + getCatServiceText(service) + "\n";
                txtCat += "  Result:" + "\n" + makeAuthorizeResultMessage(result);
                mTextCat.append(txtCat);
            }
        });
    }

    @Override
    public void onCatAuthorizeCompletion(final Cat catObj, final int code, final int sequence, final int service, final AuthorizeResult result) {
        getActivity().runOnUiThread(new Runnable() {
            @Override
            public synchronized void run() {
                int oposCode = 0;
                if (code == Epos2CallbackCode.CODE_ERR_OPOSCODE) {
                    oposCode = catObj.getOposErrorCode();
                    ShowMsg.showResult(code, oposCode, mContext);
                } else {
                    ShowMsg.showResult(code, "", mContext);
                }

                String txtCat = "";
                txtCat += "OnCatAuthorizeCompletion:\n";
                txtCat += "  Sequence:" + sequence + "\n";
                txtCat += "  Service:" + getCatServiceText(service) + "\n";
                txtCat += "  Result:" + "\n" + makeAuthorizeResultMessage(result);
                mTextCat.append(txtCat);
            }
        });
    }

    @Override
    public void onCatAccessDailyLog(final Cat catObj, final int code, final int sequence, final int service, final ArrayList<DailyLog> dailyLog) {
        getActivity().runOnUiThread(new Runnable() {
            @Override
            public synchronized void run() {
                int oposCode = 0;
                if (code == Epos2CallbackCode.CODE_ERR_OPOSCODE) {
                    oposCode = catObj.getOposErrorCode();
                    ShowMsg.showResult(code, oposCode, mContext);
                } else {
                    ShowMsg.showResult(code, "", mContext);
                }

                String txtCat = "";
                txtCat += "OnCatAccessDailyLog:\n";
                txtCat += "  Sequence:" + sequence + "\n";
                txtCat += "  Service:" + getCatServiceText(service) + "\n";
                if (dailyLog != null) {
                    for (int i = 0; i < dailyLog.size(); i++) {
                        txtCat += "  Kid:" + dailyLog.get(i).getKid() + "\n";
                        txtCat += "  SalesCount:" + dailyLog.get(i).getSalesCount() + "\n";
                        txtCat += "  SalesAmount:" + dailyLog.get(i).getSalesAmount() + "\n";
                        txtCat += "  VoidCount:" + dailyLog.get(i).getVoidCount() + "\n";
                        txtCat += "  VoidAmount:" + dailyLog.get(i).getVoidAmount() + "\n";
                    }
                }
                mTextCat.append(txtCat);
            }
        });
    }

    @Override
    public void onCatCheckConnection(final Cat catObj, final int code, final String additionalSecurityInformation) {
        getActivity().runOnUiThread(new Runnable() {
            @Override
            public synchronized void run() {
                int oposCode = 0;
                if (code == Epos2CallbackCode.CODE_ERR_OPOSCODE) {
                    oposCode = catObj.getOposErrorCode();
                    ShowMsg.showResult(code, oposCode, mContext);
                } else {
                    ShowMsg.showResult(code, "", mContext);
                }

                String txtCat = "";
                txtCat += "OnCatCheckConnection:\n";

                if(additionalSecurityInformation != null) {
                    txtCat += "  additionalSecurityInformation:" + additionalSecurityInformation + "\n";
                }
                mTextCat.append(txtCat);
            }
        });
    }

    @Override
    public void onCatScanCode(final Cat catObj, final int code, final String additionalSecurityInformation) {
        getActivity().runOnUiThread(new Runnable() {
            @Override
            public synchronized void run() {
                int oposCode = 0;
                if (code == Epos2CallbackCode.CODE_ERR_OPOSCODE) {
                    oposCode = catObj.getOposErrorCode();
                    ShowMsg.showResult(code, oposCode, mContext);
                } else {
                    ShowMsg.showResult(code, "", mContext);
                }

                String txtCat = "";
                txtCat += "onCatScanCode:\n";

                if(additionalSecurityInformation != null) {
                    txtCat += "  additionalSecurityInformation:" + additionalSecurityInformation + "\n";
                }
                mTextCat.append(txtCat);
            }
        });
    }

    @Override
    public void onCatDirectIO(Cat catObj, final int eventNumber, final int data, final String string) {
        getActivity().runOnUiThread(new Runnable() {
            @Override
            public synchronized void run() {
                String text = "";
                text += "OnCatDirectIO:\n";
                text += "  EventNumber:" + eventNumber + "\n";
                text += "  Data:" + data + "\n";
                text += "  String:" + string + "\n";
                mTextCat.append(text);
            }
        });
    }
}
